﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;



namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUnivHydrologyKinematicEdgeModel 斜面流出モデルクラス</para>
	/// </summary>
    public class DistributedModelWithKinematicWave : McForecastModelBase
	{
		/// <summary> 演算データ（キャスト用） </summary>
        DistributedModelWithKinematicWaveInfo mInf = null; // 便利の為、キャスト用に定義しておく
		
		// =========================
		// 演算実行前処理関連メソッド
		// =========================


		/// <summary><para>method outline:</para>
		/// <para>入力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "ReceiveConnectionCheck", "<<Start>>");

			bool bRtn = true;

			if (lInputDataNum == 0)
			{
				// 入力が無い場合 エラー
                csErrorInf.AddCheckErrorData(this.GetID(), DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND, "No receive data.");
				bRtn = false;
			}
			else if (lInputDataNum > 0)
			{
				for (long lP = 0; lP < lInputDataNum; lP++)
				{   // 入力データ数分

                    if ((DistributedModelWithKinematicWaveDefine.IN_EVPDATA_2D.Equals(csInputCellData[lP].GetReceivePatternID()))
                     || (DistributedModelWithKinematicWaveDefine.IN_RAINDATA_2D.Equals(csInputCellData[lP].GetReceivePatternID()))
                     || (DistributedModelWithKinematicWaveDefine.IN_RAINDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID()))
                     || (DistributedModelWithKinematicWaveDefine.IN_EVPDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID())))
					{
						// 問題なし
					}
					else
					{
						// 予期しない接続

                        csErrorInf.AddCheckErrorData(this.GetID(), DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND,
							"Unexpected receive data type. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}
			return bRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SendConnectionCheck", "<<Start>>");

			bool bRtn = true;

			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
                csErrorInf.AddCheckWarningData(this.GetID(), DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND, "No send port.");
			}
			else if (lOutputDataNum > 0)
			{
				// 入力データ数分
				for (long lP = 0; lP < lOutputDataNum; lP++)
				{
                    if (DistributedModelWithKinematicWaveDefine.OUT_PATTERN_BOTTOM.Equals(csOutputCellData[lP].GetSendPatternID()) == true)
					{
						// 問題なし
					}
					else
					{
						// 予期しない接続
                        csErrorInf.AddCheckErrorData(this.GetID(), DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND,
							"Unexpected send data type. (Send To " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}

			return bRtn;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");
			// 引数で与えられたデータを　キャストしています。
			McInitialInfo csInDt = csInitialData as McInitialInfo;

			if (csInDt != null)
			{
				csInDt.GetInfo("m_edgeDisFileName", ref mInf.m_edgeDisFileName);

				long numberOfEdges = 0;
				string sLine = "";
				int initAllNode = 0;


				try
				{
					HySFile fpParam = new HySFile(mInf.m_edgeDisFileName);
					if(fpParam.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR) < 0)
					{
						throw new Exception("No File.");
					}

					while (true)
					{
						long lRtn = fpParam.ReadText(ref sLine);
						if (lRtn == 0 || lRtn == -1)
						{
							// ファイルの終点に達した
							break;
						}
						// コメント行である場合は飛ばす
						if (sLine[0] != '#')
						{
							if (sLine == "[Data Format Name]")
							{
								fpParam.ReadText(ref sLine);
								if (sLine == "KsEdgeFc Initial State Format")
								{
									// 各エッジの上流端の初期値を与える場合
									initAllNode = 0;
								}
								else if (sLine == "KsEdgeFc Initial All State Format")
								{
									// 各ノードの初期値を与える場合
									initAllNode = 1;
								}
								else
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "データフォーマットが読み取れません");
									return false;
								}
							}

							if (sLine == "[Number of Total Edges]")
							{
								fpParam.ReadText(ref sLine);
								numberOfEdges = long.Parse(sLine);
								if (numberOfEdges <= 0)
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "エッジの数が不正です");
									return false;
								}
							}

							if (sLine == "[Data]")
							{
								for (long i = 0; i < numberOfEdges; i++)
								{
									fpParam.ReadText(ref sLine);
									String[] str = sLine.Split(new Char[] { ' ', ':' }, StringSplitOptions.RemoveEmptyEntries);


									long num;
									double updis, lowdis;
									int wn = mInf.edgeArray[i].N();
									double[] disAll = new double[wn + 1];
									int edgenum = 0;
									if (initAllNode == 0)
									{
										num = long.Parse(str[0]);
										updis = double.Parse(str[1]);
										lowdis = double.Parse(str[2]);
										edgenum = mInf.GetKey(num);
										mInf.edgeArray[edgenum].Discharge(updis, lowdis);
									}
									else if (initAllNode == 1)
									{
										num = long.Parse(str[0]);
										for (int j = 0; j <= wn; j++)
										{
											disAll[j] = double.Parse(str[j + 1]);
										}
										edgenum = mInf.GetKey(num);
										mInf.edgeArray[edgenum].DischargeAllNodeInit(disAll);
									}

									mInf.edgeArray[edgenum].CalcSatArea();
									mInf.edgeArray[edgenum].CalcStorage();
								}
							}
						}
					}
					fpParam.Close();
				}
				catch (Exception e)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "FileStreamError:" + mInf.m_edgeDisFileName + " Line:" + sLine);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "Message:" + e.Message);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "StackTrace:" + e.StackTrace);
					return false;
				
				}

				// 雨量データを取って来て、行と列の個数を調べる

				// 細かいセルの数
				long xCellNum = 0; // x 方向
				long yCellNum = 0; // y 方向
				long zCellNum = 0; // z 方向
				long idxCell = 0;

				// 雨のデータを受け取る
				for (long lP = 0; lP < lInputDataNum; lP++)
				{   // 入力データ数分。といっても雨は1データしかないはず
                    if (DistributedModelWithKinematicWaveDefine.IN_RAINDATA_2D.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						csInputCellData[lP].GetDimension(ref xCellNum, ref yCellNum, ref zCellNum, ref idxCell);

						for (long m = 0; m < mInf.numberOfEdges; m++)
						{
							// edgeArray[m] の位置を検索する
							double x = mInf.edgeArray[m].X();
							double y = mInf.edgeArray[m].Y();
							double xl, xu, yl, yu;
							long I;
							long J;

							for (I = 0; I < xCellNum; I++)
							{
								xl = mInf.m_xorig + (double)I * mInf.m_dx;
								xu = mInf.m_xorig + (double)(I + 1) * mInf.m_dx;

								for (J = 0; J < yCellNum; J++)
								{
									yl = mInf.m_yorig + (double)J * mInf.m_dy;
									yu = mInf.m_yorig + (double)(J + 1) * mInf.m_dy;

									if (xl <= x && x < xu && yl <= y && y < yu)
									{
										mInf.edgeArray[m].Gx(I);

										// Gyの値はデータ入力の関係上、上下逆にしておく
										mInf.edgeArray[m].Gy((yCellNum - 1) - J);
										break;
									}
								}

								if (J < yCellNum) break;

								if (I == xCellNum - 1)
								{
									// エラー
								}
							}
						}
					}

				}


				mInf.mQoutn = 0.0;

				// 飽和域の面積と貯留量を計算
				double sarea = 0.0;
				double storage = 0.0;
				double sumevp = 0.0;
				for (long i = 0; i < mInf.numberOfEdges; i++)
				{
					// もし edgeArray[i] が最下流エッジならば流出量を足し合わせる
					if (mInf.edgeArray[i].LowestEdge() == true)
					{
						mInf.mQoutn += mInf.edgeArray[i].DischargeLowerEnd();
					}

					if (mInf.edgeArray[i].Atr() == 1) // 流域点-流域点エッジ
					{
						sarea += mInf.edgeArray[i].Sarea();
						storage += mInf.edgeArray[i].Storage();
						sumevp += 0.0 * 0.001 / 3600.0 * mInf.edgeArray[i].Cos() * mInf.edgeArray[i].Area(); // (m3/s);
					}
				}
				mInf.mQoutc = mInf.mQoutn;

				mInf.spSarea = sarea / mInf.mSumArea;
				mInf.spStorage = storage;
				mInf.spEvp = sumevp / mInf.mSumArea * 1000.0 * 3600.0; // (mm/h)

			}
			
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = Calculate(ref csInputCellData)</para>
		/// </example>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Calculate", "<<Start>>");

			HySCellData csCell = null;
			
			// next step の状態の計算
			double all_rain = 0.0;
			double all_area = 0.0;

			mInf.mQoutn = 0.0;
			for (long i = 0; i < mInf.numberOfEdges; i++)
			{
				// edgeArray[i] に対応する地点の雨量値を取得する
				double wRainn = 0.0;
				double wEvpn = 0.0;
				// 雨のデータを受け取る
				for (long lP = 0; lP < lInputDataNum; lP++)
				{
					csInputCellData[lP].SetCurrentTime(m_csSimTime + m_csDltTime);

                    if (DistributedModelWithKinematicWaveDefine.IN_RAINDATA_2D.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						// McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Calculate",
						//	(m_csSimTime.TotalSec() - m_csStartTime.TotalSec()) + ": " + i + ": (" + 0 +", "+ i + ")-" + wRainn);
						csCell = csInputCellData[lP].GetInterpolatedCell(mInf.edgeArray[i].Gx(), mInf.edgeArray[i].Gy());
						wRainn = csInputCellData[lP].Data(csCell, 0);
					}
                    else if (DistributedModelWithKinematicWaveDefine.IN_EVPDATA_2D.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						// edgeArray[i] に対応する地点の基準蒸発散量を取得する
						// 単位時間基準蒸発散量 (m3/s) ここで　座標によって取得する雨量を変えればよい
						csCell = csInputCellData[lP].GetInterpolatedCell(mInf.edgeArray[i].Gx(), mInf.edgeArray[i].Gy());
						wEvpn = csInputCellData[lP].Data(csCell, 0);
					}
                    else if (DistributedModelWithKinematicWaveDefine.IN_RAINDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						csCell = csInputCellData[lP].GetInterpolatedCell(0);
						wRainn = csInputCellData[lP].Data(csCell, 0);
					}
                    else if (DistributedModelWithKinematicWaveDefine.IN_EVPDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						csCell = csInputCellData[lP].GetInterpolatedCell(0);
						wEvpn = csInputCellData[lP].Data(csCell, 0);
					}
				}

				all_rain = all_rain + wRainn * mInf.edgeArray[i].Area();
				all_area = all_area + mInf.edgeArray[i].Area();

				// 流出計算
				if (mInf.edgeArray[i].Calculate(wRainn, wEvpn) == false)
				{
					// エラー
				}

				// もし edgeArray[i] が最下流エッジならば流出量を足し合わせる
				if (mInf.edgeArray[i].LowestEdge() == true)
				{
					mInf.mQoutn += mInf.edgeArray[i].DischargeLowerEnd();
				}
			}

			// 飽和域の面積と貯留量を計算
			double sarea = 0.0;
			double storage = 0.0;
			double sumevp = 0.0;
			for (long i = 0; i < mInf.numberOfEdges; i++)
			{
				if (mInf.edgeArray[i].Atr() == 1) // 流域点-流域点エッジ
				{
					sarea += mInf.edgeArray[i].Sarea();
					storage += mInf.edgeArray[i].Storage();
					sumevp += mInf.edgeArray[i].ActEvp() * 0.001 / 3600.0 * mInf.edgeArray[i].Cos() * mInf.edgeArray[i].Area(); // (m3/s);
				}
			}
			
			double time_step = this.GetDeltaTime().TotalSec();
			// これまでの総流出量
			mInf.mSumQout += (mInf.mQoutc + mInf.mQoutn) * (double)time_step * 0.5;

			// 	// 現時点での貯留量
			mInf.mStorage = mInf.mSumRain - mInf.mSumQout;

			mInf.mQoutc = mInf.mQoutn;

			mInf.spSarea = sarea / mInf.mSumArea;
			mInf.spStorage = storage;
			mInf.spEvp = sumevp / mInf.mSumArea * 1000.0 * 3600.0; // (mm/h)
			
			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");
			
			HySCellData[] csSndCellData = null;
			// 最終セルだけの出力
			for (long lP = 0; lP < lOutputDataNum; lP++)
			{

				csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
				csSndCellData[0].m_dData[0] = mInf.mQoutn;
				csSndCellData[0].m_dData[1] = mInf.spSarea;
				csSndCellData[0].m_dData[2] = mInf.spStorage;
				csSndCellData[0].m_dData[3] = mInf.spEvp;

			}
			
			return 0;
		}

		// ====================
		// その他必要なメソッド
		// ====================

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			// 使用しやすいようにキャストしておく
            mInf = (DistributedModelWithKinematicWaveInfo)m_csCalInfo;

			// プロパティ設定
			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;

			if (csPrptyInfo != null)
			{
				// 演算ステップ時刻設定
				this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());

				csPrptyInfo.GetInfo("m_s1file", ref mInf.m_s1file);
				csPrptyInfo.GetInfo("m_s2file", ref mInf.m_s2file);

				csPrptyInfo.GetInfo("m_xorig", ref mInf.m_xorig);
				csPrptyInfo.GetInfo("m_yorig", ref mInf.m_yorig);
				csPrptyInfo.GetInfo("m_dx", ref mInf.m_dx);
				csPrptyInfo.GetInfo("m_dy", ref mInf.m_dy);


				int mTopoDataType = 0;
				long numberOfEdges = 0;
				string sLine = "";

				// 入力ファイル
				try
				{

					HySFile fpEdge = new HySFile(mInf.m_s1file);
					if (fpEdge.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR) < 0)
					{
						throw new Exception("No File.");
					}

					while (true)
					{
						double totalArea = 0.0;

						long lRtn = fpEdge.ReadText(ref sLine);
						if (lRtn == 0 || lRtn == -1)
						{
							// ファイルの終点に達した
							break;
						}
						// コメント行である場合は飛ばす
						if (sLine[0] != '#')
						{
							if (sLine == "[Data Format Name]")
							{
								fpEdge.ReadText(ref sLine);
								if (sLine == "edge connection V.2")
								{
									mTopoDataType = 2;
								}
								else if (sLine == "edge connection V.3")
								{
									mTopoDataType = 3;
								}
								else if (sLine == "edge connection V.4")
								{
									mTopoDataType = 4;
								}
								else if (sLine == "edge connection V.5")
								{
									mTopoDataType = 5;
								}
								else
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "エッジ接続情報のデータフォーマットが読み取れません");
									return false;
								}
							}


							if (sLine == "[Number of Total Edges]")
							{
								fpEdge.ReadText(ref sLine);
								numberOfEdges = long.Parse(sLine);
								if (numberOfEdges <= 0)
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "エッジの数が不正です");
									return false;
								}

								mInf.InitalizeKey(numberOfEdges);
								mInf.numberOfEdges = numberOfEdges;
								mInf.edgeArray = new Edge[numberOfEdges];

							}

							if (sLine == "[Data]")
							{
								for (long i = 0; i < numberOfEdges; i++)
								{
									fpEdge.ReadText(ref sLine);
									mInf.edgeArray[i] = new Edge();
									mInf.edgeArray[i].TopographicData(sLine, mTopoDataType);
									if (mInf.edgeArray[i].Atr() == 1) // 流域点-流域点
									{
										totalArea += mInf.edgeArray[i].Area();
									}

									long num = mInf.edgeArray[i].Num();
									mInf.AddKey(num);
								}
							}
						}
					}

					fpEdge.Close();

				}
				catch (Exception e)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "FileStreamError:" + mInf.m_s1file + " Line:" + sLine);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "Message:" + e.Message);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "StackTrace:" + e.StackTrace);
					return false;
				
				}

				try
				{
					// 流れの計算に必要なパラメタ値を読み込む
					HySFile fpParam = new HySFile(mInf.m_s2file);
					if(fpParam.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR) < 0)
					{
						throw new Exception("No File.");
					}

					while (true)
					{
						long lRtn = fpParam.ReadText(ref sLine);
						if (lRtn == 0 || lRtn == -1)
						{
							// ファイルの終点に達した
							break;
						}
						// コメント行である場合は飛ばす
						if (sLine[0] != '#')
						{
							if (sLine == "[Data Format Name]")
							{
								fpParam.ReadText(ref sLine);
								if (sLine == "KsEdgeFc Parameter Format")
								{

								}
								else
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "エッジパラメータﾌフォーマットが読み取れません");
									return false;
								}
							}

							if (sLine == "[Number of Total Edges]")
							{
								fpParam.ReadText(ref sLine);
								numberOfEdges = long.Parse(sLine);
								if (numberOfEdges <= 0)
								{
									// エラー
									McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "エッジの数が不正です");
									return false;
								}
							}

							if (sLine == "[Data]")
							{
								for (long i = 0; i < numberOfEdges; i++)
								{
									fpParam.ReadText(ref sLine);
									String[] str = sLine.Split(new Char[] { ' ', ':' }, StringSplitOptions.RemoveEmptyEntries);

									long num = long.Parse(str[0]);	// エッジ番号
									double man = double.Parse(str[1]);	// マニングの粗度係数 (m-s)
									double m = double.Parse(str[2]);	// q = alpha * pow(h, m) の m
									double ka = double.Parse(str[3]);	// 透水係数 (m/s)
									double d = double.Parse(str[4]);	// 土層厚さ (m)
									double da = double.Parse(str[5]);	// A 層厚 (m)
									double dm = double.Parse(str[6]);	// マトリックス部の土層厚 (m)
									double beta = double.Parse(str[7]);	// 透水係数比
									double theta = double.Parse(str[8]);	// 空間差分の重み (-)
									double cmin = double.Parse(str[9]);	// minimun C = dQ/dA (m/s)
									double Qlevel = double.Parse(str[10]);// (mm/hour)
									double eps = double.Parse(str[11]);	// (-)
									int ndiv = Int32.Parse(str[12]);	// 空間分割数 (-)

									int edgenum = mInf.GetKey(num);

									mInf.edgeArray[edgenum].ParametersInFlowModel(man, m, ka, d, da, dm, beta, theta, cmin,
												 Qlevel, eps, ndiv, (int)(m_csDltTime.TotalSec()));
								}
							}
						}
					}

					fpParam.Close();


				}
				catch (Exception e)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "FileStreamError:" + mInf.m_s2file + " Line:" + sLine);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "Message:" + e.Message);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "StackTrace:" + e.StackTrace);
					return false;
				
				}

				// upperEdgeArray を作る
				for (long i = 0; i < numberOfEdges; i++)
				{
					int upnum = mInf.edgeArray[i].NumberOfUpperEdges();
					for (int j = 0; j < upnum; j++)
					{
						long num = Math.Abs(mInf.edgeArray[i].UpperEdgeNumArray(j));
						int edgenum = mInf.GetKey(num);

						mInf.edgeArray[i].RegisterUpperEdge(mInf.edgeArray[edgenum]);
					}
				}

				// もし地形データが edge connection V.2 あるいは V.4 ならば、
				// エッジに DistributionRate, CalcUpperArea させる
				// edge connection V.3 もしくは V.5 ならば mSumArea だけ計算する
				mInf.mSumArea = 0.0;

				if (mTopoDataType == 2 || mTopoDataType == 4)
				{
					for (long i = 0; i < numberOfEdges; i++)
					{
						mInf.edgeArray[i].DistributionRate();
						mInf.edgeArray[i].CalcUpperArea();
						mInf.mSumArea += mInf.edgeArray[i].Area();
					}
				}
				else if (mTopoDataType == 3 || mTopoDataType == 5)
				{
					for (long i = 0; i < numberOfEdges; i++)
					{
						mInf.mSumArea += mInf.edgeArray[i].Area();
					}
				}

			}

			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			return true;
		}
	}
}
