﻿using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class DistributedModelWithKinematicWaveFactory : McBasicModelFactoryBase
	{
		/// <summary><para>method outline:</para>
		/// <para>デフォルトコンストラクタ</para>
		/// </summary>
		/// <param name="">無し</param>
		/// <returns>インスタンス</returns>
		/// <exception cref="">無し</exception>
        public DistributedModelWithKinematicWaveFactory()
		{
		}

		/// <summary><para>method outline:</para>
		/// <para>ファクトリ識別子を生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HySID csFactID = CreateFactoryID( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>ファクトリ識別子</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override HySID CreateFactoryID()
		{
            return DistributedModelWithKinematicWaveDefine.HYDROLOGY_MODEL_LIB;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算モデルクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalModel csCalModel = CreateCalModel(csModelKind) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>演算モデルクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalModel CreateCalModel(HySObjectKind csModelKind)
		{
			McCalModel csCalModel = null;

            if (csModelKind == DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND)
			{
				csCalModel = new DistributedModelWithKinematicWave();
			}
			else
			{
			}
			
			return csCalModel;
		}

		/// <summary><para>method outline:</para>
		/// <para>要素内演算中データクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalInfo csCalModel = CreateCalInfo( csModelKind ) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>要素内演算中データクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalInfo CreateCalInfo(HySObjectKind csModelKind)
		{
			McCalInfo csCalInfoData = null;

            if (csModelKind == DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND)
			{
                csCalInfoData = new DistributedModelWithKinematicWaveInfo();
			}
			else
			{
			}

			return csCalInfoData;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルプロパティ表示／設定情報クラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csPropertyInf = CreateModelPropertyInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>モデルプロパティ表示／設定情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCellModelPropertyIF CreateModelProperty(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McCellModelPropertyInfo csRtnCellPrptyDt = null;

            if (csModelKind == DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
				csRtnCellPrptyDt.SetStepTime(600.0);

				csRtnCellPrptyDt.AddInfoType("m_s1file", "Edge connection ファイル", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_s1file", "./HywrModelSample/EdgeSample.dat");

				csRtnCellPrptyDt.AddInfoType("m_s2file", "パラメータファイル", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_s2file", "./HywrModelSample/EdgePmtSample.dat");
				csRtnCellPrptyDt.AddInfoType("m_xorig", "2次元降雨データの原点X座標 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_xorig", -2970.91);
				csRtnCellPrptyDt.AddInfoType("m_yorig", "2次元降雨データの原点Y座標 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_yorig", -3007.172);
				csRtnCellPrptyDt.AddInfoType("m_dx", "2次元降雨データの空間X軸間隔 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dx", 2500.0);
				csRtnCellPrptyDt.AddInfoType("m_dy", "2次元降雨データの空間Y軸間隔 (m)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dy", 2500.0);

				
				// 受信可能なパターン
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                    DistributedModelWithKinematicWaveDefine.IN_RAINDATA_2D,
					McTranInfoDefine.D2_CELL_SERIAL,
					"二次元雨量") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "単位時間雨量強度", HySDataKindDefine.SCALOR_RAIN_FALL, "mm/hour");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
					csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加

				// テスト用に一様な雨量を受信できるようにしておく。
                csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(DistributedModelWithKinematicWaveDefine.IN_RAINDATA_POINT,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "一様雨量") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "単位時間雨量強度", HySDataKindDefine.SCALOR_RAIN_FALL, "mm/hour");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
					csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加



				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                    DistributedModelWithKinematicWaveDefine.IN_EVPDATA_2D,
					McTranInfoDefine.D2_CELL_SERIAL,
					"二次元蒸発散量") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "単位時間蒸発散強度", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "mm/hour"); // 最初の変数は流量
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
					csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加

				// テスト用に一様な雨量を受信できるようにしておく。
                csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(DistributedModelWithKinematicWaveDefine.IN_EVPDATA_POINT,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "一様蒸発散量") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "単位時間蒸発散強度", HySDataKindDefine.SCALOR_RAIN_FALL, "mm/hour");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
					csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加


				// 送信パターン
				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
                    DistributedModelWithKinematicWaveDefine.OUT_PATTERN_BOTTOM,
					McTranInfoDefine.SINGLE_CELL_SERIAL,
					"下流端出力") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(4);
					{
						csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec");
						csCellChara.SetDataKind(1, "飽和面積", HySDataKindDefine.ANY_VALUE, "m2");
						csCellChara.SetDataKind(2, "貯水量", HySDataKindDefine.WATER_VOLUME, "m3");
						csCellChara.SetDataKind(3, "蒸発散量", HySDataKindDefine.ANY_VALUE, "m3");

					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}

				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
			
			}
			else
			{
				// Do nothing
			}

			return csRtnCellPrptyDt;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル初期化表示／設定情報生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csInitialInf = CreateModelInitialInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>初期情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McPropertyInfoRoot CreateModelInitialInfo(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McInitialInfo csRtnDt = null;

            if (csModelKind == DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);

				csRtnDt.AddInfoType("m_edgeDisFileName", "初期状態設定ファイル名", McDefine.ValKind.STRING);
				csRtnDt.SetInfo("m_edgeDisFileName", "./HywrModelSample/EdgeIstSample.dat");
			}
			else
			{
				// Do Nothing
			}

			return csRtnDt;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル情報提供</para>
		/// </summary>
		/// <example>
		/// <para>usage:</para>
		/// <para>HySDataLinkedList csModelInfoList = GetCalModelInfoList();</para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>モデル情報のリスト</returns>
		/// <exception cref="">無し</exception>
		/// <remarks>
		/// <para>remarks:</para>
		/// <para>モデル情報(McModelInfo)をリスト形式で返却する</para>
		/// </remarks>
		public override HySDataLinkedList GetCalModelInfoList()
		{
			McModelInfo csModelInfo=null;

			csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
					McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY,
                    DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_KIND,
                    DistributedModelWithKinematicWaveDefine.HYDROLOGY_KINEMATIC_EDGE_NAME);
			csModelInfo.SetVersionInf("Ver1.0 ");
			csModelInfo.SetSummaryInf("キネマティックウェーブモデルを適用した分布型斜面流出モデル");
			csModelInfo.SetCreatorInf("京都大学大学院 水文・水資源学研究室");
			csModelInfo.SetIconName("Basin");
			m_csCalModelInforList.AddLast(csModelInfo);

			return m_csCalModelInforList;
		}

	}
}


namespace CommonMP.HYMCO._SYSTEM_
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class HYMCO_MODEL_FACTORY_CREATOR_IMPLE : HYMCO_MODEL_FACTORY_CREATOR
	{
		/// <summary><para>method outline:</para>
		/// <para>コンストラクター</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HYMCO_MODEL_FACTORY_CREATOR_IMPLE csFactCrt = new HYMCO_MODEL_FACTORY_CREATOR_IMPLE( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>生成インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public HYMCO_MODEL_FACTORY_CREATOR_IMPLE()
		{
		}
		/// <summary><para>method outline:</para>
		/// <para>演算モデルファクトリクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McModelFactory csCalModelFactory = CreateFactory( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>演算モデルファクトリクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public CommonMP.HYMCO.Interface.Model.McModelFactory CreateFactory()
		{
            return new KyotoUnivEngHywr.DistributedModelWithKinematicWaveFactory();
		}
	}
}