﻿
using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

using CommonMP.HYMCO.CoreImpl.Data;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUnivHydrologyKinematicEdgeModel 斜面流出データクラス</para>
	/// </summary>
	[Serializable]
	public class DistributedModelWithKinematicWaveInfo : McCalInfo
	{
		/// <summary>斜面素片(エッジ)の数</summary>
		public long numberOfEdges;

		/// <summary>計算時間間隔</summary>
		public int mDt;

		/// <summary>Edge Connectionファイル </summary>
		public string m_s1file;

		/// <summary>透水係数等が書かれたファイル </summary>
		public string m_s2file;

		/// <summary>2次元雨量データの原点</summary>
		public double m_xorig, m_yorig;
		/// <summary>2次元雨量データのX軸格子方向の大きさ</summary>
		public double m_dx;
		/// <summary>2次元雨量データのY軸格子方向の大きさ</summary>
		public double m_dy;

		/// <summary>斜面素片の配列</summary>
		public Edge[] edgeArray;

		/// <summary>current_time での最下流端 Edge からの流出量の総和 (m3/s)</summary>
		public double mQoutc;
		/// <summary>next_time での最下流端 Edge からの流出量の総和 (m3/s)</summary>
		public double mQoutn;

				/// <summary>総降水量 (m3)</summary>
		public double mSumRain;
		/// <summary>総流出量 (m3)</summary>
		public double mSumQout;
		/// <summary>流域総面積 (m2)</summary>
		public double mSumArea;
		/// <summary> 貯留量 (m3) = mSumRain - mSumQout</summary>
		public double mStorage;
		/// <summary>総蒸発散量 (m3)</summary>
		public double mSumEvp;

		/// <summary>
		/// 出力する状態量だけ別に用意する
		/// </summary>
		public double spSarea, spStorage, spEvp;

		/// <summary>エッジの上下端の流量を記したファイルの名前</summary>
		public string m_edgeDisFileName;

		/// <summary><para>斜面地形データの種類を表す変数</para>
		/// <para>この値によって Set_parameter の動作が変わる</para>
		/// <para>1: edge connection V.1 データセット</para>
		/// <para>2: edge connection V.2 データセット</para>
		/// <para>3: edge connection V.3 データセット</para>
		/// <para>4: edge connection V.4 データセット</para>
		/// <para>5: edge connection V.5 データセット</para>
		/// </summary>
		public int mTopoDataType;

		/// <summary>
		/// バイナリツリー構造をつくるのは後回しにして全検索でサボる
		/// </summary>
		private long[] m_KeyStock;
		private int m_NumKeyStock = 0;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Key"></param>
		/// <returns></returns>
		public int AddKey(long Key)
		{
			m_KeyStock[m_NumKeyStock++] = Key;
			return m_NumKeyStock - 1;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Key"></param>
		/// <returns></returns>
		public int GetKey(long Key)
		{
			for (int i = 0; i < m_NumKeyStock; i++)
			{
				if (Key == m_KeyStock[i])
				{
					return i;
				}
			}
			// エラー
			return -1;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="MaxRange"></param>
		public void InitalizeKey(long MaxRange)
		{
			m_NumKeyStock = 0;
			m_KeyStock = new long[MaxRange];
		}


		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
            DistributedModelWithKinematicWaveInfo csRtn = new DistributedModelWithKinematicWaveInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
            DistributedModelWithKinematicWaveInfo csOrgDt = (DistributedModelWithKinematicWaveInfo)csOrgInfo;

			// 情報コピーはできない
			return false;
		}

	}

	/// <summary>
	/// 斜面素片クラス
	/// </summary>
	public class Edge
	{
		// エッジの基本的な諸元

		/// <summary>エッジ番号</summary>
		protected long mNum;

		/// <summary>エッジ中点の座標値 (m)</summary>
		protected double mX, mY, mZ;

		/// <summary>エッジの属性</summary>
		protected int mAtr;
		// 1:流域点-流域点
		// 2:流域点-河道点

		/// <summary>エッジの長さ(3次元) (m)</summary>
		protected double mL;

		/// <summary>
		/// エッジの面積 (m2)
		/// </summary>
		protected double mArea;

		/// <summary>エッジの幅(m)</summary>
		protected double mWidth;

		/// <summary>エッジの勾配(sin) (-) </summary>
		protected double mGrd;

		/// <summary>cos(asin(mGrd)) (-)</summary>
		protected double mCos;

		/// <summary>上流に位置するエッジの数</summary>
		protected int numberOfUpperEdges;
		/// <summary>上流に位置するエッジの番号の配列へのポインタ</summary>
		protected long[] upperEdgeNumArray;

		/// <summary>上流に位置するエッジへのアドレスを記憶する配列へのポインタ</summary>
		protected Edge[] upperEdgeArray;

		/// <summary>下流に位置するエッジの数</summary>
		protected int numberOfLowerEdges;
		/// <summary>流に位置するエッジの番号の配列へのポインタ</summary>
		protected long[] lowerEdgeNumArray;
		/// <summary>下流に位置するエッジへのアドレスを記憶する配列へのポインタ</summary>
		protected Edge[] lowerEdgeArray;

		/// <summary>属性が 2 でかつ下流側流出エッジが存在しないとき YES, それ以外は NO</summary>
		protected bool lowestEdge;

		/// <summary>上流域の面積 (m2)</summary>
		protected double mUpperArea;

		/// <summary>斜面素片中点が含まれる格子セルの座標値</summary>
		protected long mGx, mGy;

		/// <summary>斜面の番号</summary>
		public long Num() { return mNum; }
		/// <summary>斜面の属性</summary>
		public int Atr() { return mAtr; }
		/// <summary>斜面長(m)</summary>
		public double Length() { return mL; }
		/// <summary>斜面面積(m2)</summary>
		public double Area() { return mArea; }
		/// <summary>斜面幅(m)</summary>
		public double Width() { return mWidth; }
		/// <summary>斜面勾配(Sin)</summary>
		public double Gradient() { return mGrd; }
		/// <summary>斜面勾配(Cos)</summary>
		public double Cos() { return mCos; }
		/// <summary>上流に位置するエッジの数</summary>
		public int NumberOfUpperEdges() { return numberOfUpperEdges; }
		/// <summary>斜面が最下流エッジであるか</summary>
		public bool LowestEdge() { return lowestEdge; }
		/// <summary>斜面の上流域の面積</summary>
		public double UpperArea() { return mUpperArea; }

		/// <summary>斜面の中点の座標値(m)</summary>
		public double X() { return mX; }
		/// <summary>斜面の中点の座標値(m)</summary>
		public double Y() { return mY; }
		/// <summary>斜面の中点の座標値(m)</summary>
		public double Z() { return mZ; }

		/// <summary>斜面素片中点が含まれる格子セルの座標値(m)</summary>
		public long Gx() { return mGx; }
		/// <summary>斜面素片中点が含まれる格子セルの座標値(m)</summary>
		public long Gy() { return mGy; }
		/// <summary>斜面素片中点が含まれる格子セルの座標値を設定する</summary>
		public void Gx(long wgx) { mGx = wgx; }
		/// <summary>斜面素片中点が含まれる格子セルの座標値を設定する</summary>
		public void Gy(long wgy) { mGy = wgy; }

		/// <summary>上流に位置するエッジ番号へのポインタを呼ぶ</summary>
		public long UpperEdgeNumArray(int i) { return upperEdgeNumArray[i]; }

		/// <summary>空間差分分割数</summary>
		public int N() { return mN; }

		/// <summary>斜面素片を上流斜面として登録する</summary>
		public bool RegisterUpperEdge(Edge p)
		{
			for (int i = 0; i < numberOfUpperEdges; i++)
			{
				if (upperEdgeArray[i] == null)
				{
					upperEdgeArray[i] = p;
					return true;
				}
			}

			return false;
		}

		/// <summary>マニングの粗度係数 (m-s)</summary>
		protected double mMan;
		/// <summary>透水係数 (m/s)</summary>
		protected double mKa;
		/// <summary>土層厚さ (m)</summary>
		protected double mD;
		/// <summary> A 層厚(土層に空隙率を掛た後の値) (m)</summary>
		protected double mDa;
		/// <summary>マトリックス部に相当する土層厚 (m)</summary>
		protected double mDm;
		/// <summary>透水係数比 (ka/km)</summary>
		protected double mBeta;

		/// <summary>kinematic wave routing parameter (m-s)</summary>
		protected double mAlpha;
		/// <summary>重力水部分の流速 (m/s)</summary>
		protected double mVa;
		/// <summary>kinematic wave routing parameter (-)</summary>
		protected double mM;

		/// <summary>空間差分分割数 (-)</summary>
		public int mN;
		/// <summary>time step (s)</summary>
		protected int mDt;

		/// <summary><para>time weighting parameter (-). </para>
		/// <para>The standard value of mTheta is 0.65.</para>
		/// <para>If mTheta = 0, the finite difference calculation scheme becomes an explicit one.</para>
		/// </summary>
		protected double mTheta;

		/// <summary><para>minimum C = dQ/dA. </para>
		/// <para>If the calculated value of dQ/dA is smaller than cmin, the value is replaced with mCmin.</para>
		/// </summary>
		protected double mCmin;


		/// <summary><para>Reference level of discharge calculation (mm/hour) </para>
		/// <para>The standard value is 10.0 (mm/hour).</para>
		/// </summary>
		protected double mQlevel;

		/// <summary><para>tolerance level of diffrence of two consecutive caluculateddischarges to stop the iteration (-).</para>
		/// <para> The standard value is  0.001</para>
		/// </summary>
		protected double mEps;

		/// <summary>流量分配率 (-)</summary>
		protected double mDist;

		/// <summary>current_time での流量 (m3/s)</summary>
		protected double[] mQc;	// 

		// 作業用変数
		/// <summary>length of the division. mDx = mL/mN (m)</summary>
		protected double mDx;
		/// <summary>next_time での流量 (m3/s)</summary>
		public double[] mQn; 
		/// <summary>"small" discharge valuse (m3/s)</summary>
		protected double mQsmall; 

		/// <summary>飽和面積 (m2)</summary>
		protected double mSarea;
		/// <summary>斜面素片内の貯留量 (m3)</summary>
		protected double mStorage;
		/// <summary>斜面素片内の平均水位 (m) : Storage() で計算</summary>
		protected double mHaverage;
		/// <summary>斜面素片内からの実蒸発散量 (mm/h)</summary>
		protected double mActEvp;

		/// <summary>コンストラクタ</summary>
		public Edge()
		{
			mL = 0.0; mGrd = 0.0; mArea = 0.0; mWidth = 0.0;
			mGx = 0; mGy = 0;
		}

		/// <summary>
		/// エッジの計算順に並べられた edge connection V.{2/3/4/5} ファイルを読み込む
		/// </summary>
		/// <param name="fpstring">エッジ情報の記載された一行の文字列</param>
		/// <param name="wTopoDataType">エッジ情報のバージョン</param>
		public void TopographicData(String fpstring, int wTopoDataType)
		{
			// エッジ情報を読み込むストリング配列
			String[] getline;

			// 現在読み込み中のストリング番号（読み込むたびに前進する）
			int iread = 0;

			getline = fpstring.Split(new Char[] { ' ', ':' }, StringSplitOptions.RemoveEmptyEntries);

			mNum = long.Parse(getline[iread++]);

			// エッジ中点の座標を読み込む(V.2/3)か、
			// エッジ上下端のノード番号とそれぞれの座標値を読み込み、
			// エッジ中点の座標を計算する(V.4/5)。
			if (wTopoDataType == 2 || wTopoDataType == 3)
			{
				mX = double.Parse(getline[iread++]);
				mY = double.Parse(getline[iread++]);
				mZ = double.Parse(getline[iread++]);

				// fscanf(fpTopo, "%lf %lf %lf", &mX, &mY, &mZ);
			}
			else if (wTopoDataType == 4 || wTopoDataType == 5)
			{
				long wNum;
				double wUpX, wUpY, wUpZ, wLowX, wLowY, wLowZ;
				wNum = long.Parse(getline[iread++]);
				wUpX = double.Parse(getline[iread++]);
				wUpY = double.Parse(getline[iread++]);
				wUpZ = double.Parse(getline[iread++]);

				wNum = long.Parse(getline[iread++]);
				wLowX = double.Parse(getline[iread++]);
				wLowY = double.Parse(getline[iread++]);
				wLowZ = double.Parse(getline[iread++]);

				mX = 0.5 * (wUpX + wLowX);
				mY = 0.5 * (wUpY + wLowY);
				mZ = 0.5 * (wUpZ + wLowZ);
			}
			else
			{
				// エラー
			}

			// 上流側エッジの数を読み込む(共通)
			numberOfUpperEdges = int.Parse(getline[iread++]);

			// upperEdgeNumArray, upperEdgeArray の確保
			if (numberOfUpperEdges != 0)
			{
				upperEdgeNumArray = new long[numberOfUpperEdges];
				upperEdgeArray = new Edge[numberOfUpperEdges];

				// upperEdgeArray の初期化
				for (int i = 0; i < numberOfUpperEdges; i++) upperEdgeArray[i] = null;

				// 上流側エッジの番号を読み込む(共通)
				for (int i = 0; i < numberOfUpperEdges; i++)
				{
					upperEdgeNumArray[i] = long.Parse(getline[iread++]);
				}
			}			

			// 下流側エッジの数を読み込む(共通)
			int tmp = int.Parse(getline[iread++]);

			// 下流側エッジの番号を読み込む(共通)
			bool outEdgeFromLowerEnd = false;
			// 下端から流出するエッジがある場合
			for (int i = 0; i < tmp; i++)
			{
				if (long.Parse(getline[iread++]) < 0) outEdgeFromLowerEnd = true;
			}

			// エッジの属性を読み込む(共通)
			mAtr = int.Parse(getline[iread++]);

			// 流域最下端のエッジかどうか確認する
			if (mAtr == 2 && outEdgeFromLowerEnd == false) // 最下端エッジ
			{
				lowestEdge = true;
			}
			else
			{
				lowestEdge = false;
			}

			// mAtr が 1 のとき(流域点-流域点エッジのとき)
			// エッジの長さ、勾配、面積を読み込む(共通)
			if (mAtr == 1)
			{
				double wlen, wgrd, warea;
				wlen = double.Parse(getline[iread++]);
				wgrd = double.Parse(getline[iread++]);
				warea = double.Parse(getline[iread++]);

				mArea = warea;
				mWidth = mArea / wlen;

				// wlen は水平距離なので3次元長さに変換する
				mL = wlen * Math.Sqrt(1.0 + wgrd * wgrd);

				// wgrd は tangent の値なので、sin の値に変換する
				wgrd = Math.Atan(wgrd);
				mGrd = Math.Sin(wgrd);

				// mCos を求める。mCos は降水強度に乗じる
				mCos = Math.Cos(Math.Asin(mGrd));
			}

			if (wTopoDataType == 3 || wTopoDataType == 5)
			{
				mDist = double.Parse(getline[iread++]);
				mUpperArea = double.Parse(getline[iread++]);
			}
		}

		/// <summary>
		/// 流れの計算に必要なパラメタ値を読み込む
		/// </summary>
		/// <param name="man">マニングの粗度係数 (m-s)</param>
		/// <param name="m">q = alpha * pow(h, m) の m</param>
		/// <param name="ka">透水係数 (m/s)</param>
		/// <param name="d">土層厚さ (m)</param>
		/// <param name="da">A 層厚 (m)</param>
		/// <param name="dm">マトリックス部の土層厚 (m)</param>
		/// <param name="beta">透水係数比</param>
		/// <param name="theta"> </param>
		/// <param name="cmin">minimum C = dQ/dA</param>
		/// <param name="Qlevel"></param>
		/// <param name="eps"></param>
		/// <param name="ndiv">空間差分分割数 (-)</param>
		/// <param name="dt">time_step (s)</param>
		public void ParametersInFlowModel(double man, double m, double ka, double d,
		 double da, double dm, double beta, double theta, double cmin, double Qlevel, double eps, int ndiv, int dt)
		{
			mMan = man;
			mM = m;
			mKa = ka;
			mD = d;
			mDa = da;
			mDm = dm;
			mBeta = beta;
			mTheta = theta;
			mCmin = cmin;
			mQlevel = Qlevel;
			mEps = eps;
			mN = ndiv;
			mDt = dt;

			mQc = new double[mN + 1];
			mQn = new double[mN + 1];

			if (mAtr == 1)
			{
				mAlpha = Math.Sqrt(mGrd) / mMan;
				mVa = mKa * mGrd;
				mDx = mL / (double)mN;
			}

			mQsmall = mQlevel * mEps * 0.001 / 3600.0;
			// mQlevel: 流量の計算値の基準値 (mm/hour). 10.0 が標準値.
			// mEps: 計算の精度 (-). 0.001 が標準値
		}

		/// <summary>
		/// current_time での上下端の流量を与え、内部の計算断面の流量を線形補間で求める
		/// </summary>
		/// <param name="disUpperEnd"></param>
		/// <param name="disLowerEnd"></param>
		public void Discharge(double disUpperEnd, double disLowerEnd)
		{
			mQc[0] = mQn[0] = disUpperEnd;
			mQc[mN] = mQn[mN] = disLowerEnd;

			// 内部の計算断面の流量を線形補間で求める
			for (int i = 1; i <= mN - 1; i++)
			{
				mQc[i] = mQn[i] = mQc[0] +
				(mQc[mN] - mQc[0]) / (double)mN * (double)i;
			}

		}

		/// <summary>
		/// current_time での上端の流量を返す
		/// </summary>
		/// <returns></returns>
		public double Discharge_UpperEnd()
		{
			return mQc[0];
		}

		/// <summary>
		/// current_time での下端の流量を返す
		/// </summary>
		/// <returns></returns>
		public double Discharge_LowerEnd()
		{
			return mQc[mN];

		}

		/// <summary>
		/// current_time での全ノードの流量を与える
		/// </summary>
		/// <param name="disAllNode"></param>
		public void DischargeAllNodeInit(double[] disAllNode)
		{
			for (int i = 0; i <= mN; i++)
			{
				mQc[i] = mQn[i] = disAllNode[i];
			}
		}

		/// <summary>
		///  current_time での全ノードの流量を返す
		/// </summary>
		/// <param name="disAllNode"></param>
		public void DischargeAllNode(ref double[] disAllNode)
		{
			for (int i = 0; i <= mN; i++)
			{
				disAllNode[i] = mQn[i];
			}
		}

		/// <summary>
		/// mQn から飽和域の面積を計算する
		/// </summary>
		public void CalcSatArea()
		{
			// 飽和域の面積の計算
			mSarea = 0.0;
			double wQsat = mVa * mWidth * (mDa - mDm);
			for (int i = 1; i <= mN; i++)
			{
				// 区間上端で飽和
				if (mQn[i - 1] >= wQsat)
				{
					// 区間下端でも飽和
					if (mQn[i] >= wQsat)
						mSarea += mWidth * mDx;
					// 下端では不飽和
					else
						mSarea += mWidth * mDx *
							(wQsat - mQn[i - 1]) / (mQn[i] - mQn[i - 1]);
				}
				// 上端では不飽和
				else
				{
					// 下端では飽和
					if (mQn[i] >= wQsat)
						mSarea += mWidth *
							(mDx - mDx * (wQsat - mQn[i - 1]) / (mQn[i] - mQn[i - 1]));
					// 下端では不飽和
					else
						mSarea += 0.0;
				}
			}

			mSarea *= mCos;
		}


		/// <summary>
		///  mQn から貯留量を計算する
		/// </summary>
		public void CalcStorage()
		{
			double[] wHn = new double[mN + 1];
			for (int i = 0; i <= mN; i++)
			{
				if (mWidth == 0.0)
				{
					wHn[i] = 0.0;
				}
				else if (mQn[i] < 0.00000001)
				{
					wHn[i] = 0.0;
				}
				else if (mQn[i] / mWidth < mVa / mBeta * mDm) // マトリクス部だけの場合
				{
					wHn[i] = Math.Pow((mQn[i] / mWidth) * mBeta / (mDm * mVa), 1 / mBeta) * mDm;
				}
				else if (mQn[i] / mWidth < mVa / mBeta * mDm + mVa * (mDa - mDm)) // マトリクス部＋中間流の場合
				{
					wHn[i] = mDm + (mQn[i] / mWidth - mVa / mBeta * mDm) / mVa;
				}
				else
				{
					// 初期推定値は h=mDa とする。
					double h0 =
						mDa + Math.Pow(((mQn[i] / mWidth - (mVa * (mDa - mDm) + mVa / mBeta * mDm)) / mAlpha), 1.0 / mM);
					double v0 = Math.Pow(h0 - mDa, mM - 1.0);
					double q0 = mAlpha * v0 * (h0 - mDa) + mVa * (h0 - mDm) + mVa / mBeta * mDm;
					double q1;

					while (true)
					{
						h0 = h0 - (q0 - mQn[i] / mWidth) / (mAlpha * mM * v0 + mVa);
						v0 = Math.Pow(h0 - mDa, mM - 1.0);
						q1 = mAlpha * v0 * (h0 - mDa) + mVa * (h0 - mDm) + mVa / mBeta * mDm;
						if (Math.Abs(q1 - q0) < mEps) break;
						q0 = q1;
					}
					wHn[i] = h0;
				}
			}

			mStorage = 0.0;
			mHaverage = 0.0;
			for (int i = 1; i <= mN; i++)
			{
				mStorage += (wHn[i - 1] + wHn[i]) * mDx * 0.5 * mWidth;
				// 斜面素片内の平均水位 (m)
				mHaverage += (wHn[i - 1] + wHn[i]);
			}
			mHaverage /= (double)mN;

		}

		/// <summary>
		///  evalpha : (実蒸発散量 / 基準蒸発散量) の係数を mHaverage を変数として計算する．
		/// </summary>
		/// <returns></returns>
		public double Evalpha()
		{
			double alpha;
			double cBeta = 0.3;
			double w = mHaverage / mD;// 体積含水率
			double gammac = mDm / mD; // マトリクス部の飽和体積含水率
			double cBetaw;

			double M_PI = 3.14;
			if (w < gammac)
			{
				cBetaw = 0.5 * (1.0 - Math.Cos(w / gammac * M_PI));
			}
			else
			{
				cBetaw = 1.0;
			}
			double evagamma = cBetaw * cBeta;
			alpha = 2.0 * Math.Pow(evagamma, 0.81) / (1.0 + Math.Pow(evagamma, 0.98));

			return alpha;
		}

		/// <summary>
		/// Calculate のなかで状態量の更新も行なう
		/// </summary>
		/// <param name="r">降水強度 (mm/hour)</param>
		/// <param name="e">単位時間当たり基準蒸発量 (mm/hour)</param>
		/// <returns></returns>
		public bool Calculate(double r, double e)
		{
			// upstream end
			mQn[0] = 0.0;
			for (int i = 0; i < numberOfUpperEdges; i++)
			{
				if (upperEdgeNumArray[i] > 0) // 流入エッジ
				{
					mQn[0] += upperEdgeArray[i].DischargeLowerEnd();
					double testu = upperEdgeArray[i].mX+ upperEdgeArray[i].mY;
				}
			}
			mQn[0] *= mDist;	// 流量分配率をかける

			if (mAtr == 2)	// 流域点-河道点エッジの場合
			{
				mQc[mN] = mQn[0];

				for (int i = 1; i <= mN; i++)
				{
					mQn[i] = mQn[0];
				}

				return true;
			}

			double dta = mDt;	// time step (s)

			// 基準蒸発散量にかける係数を計算．
			double evalpha = Evalpha();

			mActEvp = evalpha * e;

			// r (mm/h) から 実蒸発散量 (mm/h)を引く
			r -= mActEvp;

			// lateral inflow
			double rLin = r * 0.001 / 3600.0 * mCos; // 有効降水強度 (m/s)

			double cold; // double cnew; 
			double inflow_area = mUpperArea;

			for (int i = 1; i <= mN; i++)
			{
				double wm = mWidth;
				inflow_area += wm * mDx;
				double delta_q = mQsmall * inflow_area;
				double q = 0.5 * (mQc[i - 1] + mQc[i]) / wm;
				cold = PropagationSpeed(mAlpha, mVa, mDa, mDm, mBeta,
				 mM, delta_q / wm, q);
				if (cold < mCmin) cold = mCmin;

				mQn[i] = (mQc[i] + mTheta * cold *
				(dta / mDx * mQn[i - 1] + dta * wm * rLin) -
				(1.0 - mTheta) * cold * dta *
				((mQc[i] - mQc[i - 1]) / mDx - rLin * wm)) /
				(1.0 + mTheta * cold * dta / mDx);

				if (mQn[i] < 0.0)
				{
					mQn[i] = 0.0;
				}
			}

			// 飽和域の面積の計算
			CalcSatArea();

			CalcStorage();

			// update state variables
			for (int i = 0; i <= mN; i++)
			{
				mQc[i] = mQn[i];
			}

			return true;
		}

		/// <summary>
		/// 最下流端の流量を返す
		/// </summary>
		/// <returns></returns>
		public double DischargeLowerEnd() 
		{
			return mQc[mN]; 
		}

		/// <summary>
		/// 流量分配率 mDist を計算する
		/// </summary>
		public void DistributionRate()
		{
			if (mAtr == 2)	// 流域点-河道点エッジの場合
			{
				int couter = 1;	// 自分も含めた上流側分流エッジの数
				for (int i = 0; i < numberOfUpperEdges; i++)
				{
					if (upperEdgeNumArray[i] < 0)
					// 上流側分流エッジが存在する
					{
						couter++;
					}
				}

				mDist = 1.0 / couter;
				return;
			}

			// 流域点-流域点エッジの場合
			double tmpa, tmpb;
			tmpa = tmpb = mWidth * Math.Sqrt(mGrd);

			int counter = 1;				// 上流側分流エッジの数

			bool anyBRedge = false;
			// 上流側分流エッジに一つでも流域点-河道点エッジが存在する場合、YES 

			for (int i = 0; i < numberOfUpperEdges; i++)
			{
				// 上流側分流エッジが存在する
				if (upperEdgeNumArray[i] < 0)
				{
					counter++;

					// 分流エッジが流域点-流域点エッジである場合
					if (upperEdgeArray[i].Atr() == 1)
					{
						tmpb += upperEdgeArray[i].Width() *
						Math.Sqrt(upperEdgeArray[i].Gradient());
					}
					else if (upperEdgeArray[i].Atr() == 2)
					{
						anyBRedge = true;
					}
				}
			}

			if (anyBRedge == false)
			{
				mDist = tmpa / tmpb;
			}
			// 上流側分流エッジに一つでも流域点-河道点エッジが存在する
			// 場合は、上の式を使えないので、単純に頭数で割る 
			else
				mDist = 1.0 / counter;
		}

		/// <summary>
		/// 上流域の面積を計算する
		/// </summary>
		public void CalcUpperArea()
		{
			mUpperArea = 0.0;
			for (int i = 0; i < numberOfUpperEdges; i++)
			{
				// printf("%ld : %ld\n", mNum, upperEdgeArray[i].Num());

				// 上流から流入するエッジが存在する
				if (upperEdgeNumArray[i] > 0)
				{
					// 流入エッジが流域点-流域点エッジである場合
					if (upperEdgeArray[i].Atr() == 1)
					{
						mUpperArea += upperEdgeArray[i].Area() +
						upperEdgeArray[i].UpperArea();
					}
					else if (upperEdgeArray[i].Atr() == 2)
					{
					}
				}
			}

			mUpperArea *= mDist;

		}

		/// <summary>
		/// 伝播速度 c を計算して返す
		/// </summary>
		/// <param name="aAlpha">パラメタ alpha </param>
		/// <param name="aVa">重力水部の流速 (m/s)</param>
		/// <param name="aDa"> A 層厚 (m)</param>
		/// <param name="aDm">マトリックス部の土層厚 (m)</param>
		/// <param name="aBeta">beta (-)</param>
		/// <param name="aM">kinematic routing parameter (-)</param>
		/// <param name="aQeps">収束判定値</param>
		/// <param name="aQunit">単位幅当りの流量</param>
		/// <returns></returns>
		public double PropagationSpeed(double aAlpha, double aVa, double aDa, double aDm, double aBeta, double aM, double aQeps, double aQunit)
		{
			if (aQunit < aVa / aBeta * aDm) // マトリクス部だけの場合
			{
				double h = Math.Pow(aQunit * aBeta / (aVa * aDm), 1.0 / aBeta) * aDm;
				return aVa * Math.Pow(h / aDm, aBeta - 1.0);
			}
			else if (aQunit < aVa * (aDa - aDm) + aVa / aBeta * aDm) // マトリクス部・中間流の場合
			{
				return aVa;
			}

			// 表面流も発生している場合
			double h0 =
				aDa + Math.Pow(((aQunit - (aVa * (aDa - aDm) + aVa / aBeta * aDm)) / aAlpha), 1.0 / aM);
			double v0 = Math.Pow(h0 - aDa, aM - 1.0);
			double q0 = aAlpha * v0 * (h0 - aDa) + aVa * (h0 - aDm) + aVa / aBeta * aDm;
			double q1;
			int count;
			count = 0;
			while (true)
			{
				h0 = h0 - (q0 - aQunit) / (aAlpha * aM * v0 + aVa);
				v0 = Math.Pow(h0 - aDa, aM - 1.0);
				q1 = aAlpha * v0 * (h0 - aDa) + aVa * (h0 - aDm) + aVa / aBeta * aDm;
				// if (q1 > q0 - aQeps) break;
				if (Math.Abs(q1 - q0) < aQeps) break;
				q0 = q1;

				if (count > 10000)
				{
					double c = 0;
					return c;
				}
				count += 1;
			}

			double cxx = aAlpha * aM * v0 + aVa;
			return cxx;
		}

		/// <summary>
		/// 斜面の飽和面積(m2)
		/// </summary>
		/// <returns></returns>
		public double Sarea() { return mSarea; }

		/// <summary>
		/// 斜面素片内の貯留量(m3)
		/// </summary>
		/// <returns></returns>
		public double Storage() { return mStorage; }

		/// <summary>
		/// 斜面素片内の実蒸発散量(m3)
		/// </summary>
		/// <returns></returns>
		public double ActEvp() { return mActEvp; }
	}

}
